# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2._list_object import ListObject
from stripe.v2.money_management._outbound_setup_intent import (
    OutboundSetupIntent,
)
from typing import cast
from typing_extensions import Literal, NotRequired, TypedDict


class OutboundSetupIntentService(StripeService):
    class CancelParams(TypedDict):
        pass

    class CreateParams(TypedDict):
        payout_method: NotRequired[str]
        """
        If provided, the existing payout method resource to link to this setup intent.
        Any payout_method_data provided is used to update information on this linked payout method resource.
        """
        payout_method_data: NotRequired[
            "OutboundSetupIntentService.CreateParamsPayoutMethodData"
        ]
        """
        If no payout_method provided, used to create the underlying credential that is set up for outbound money movement.
        If a payout_method provided, used to update data on the credential linked to this setup intent.
        """
        usage_intent: NotRequired[Literal["payment", "transfer"]]
        """
        Specify which type of outbound money movement this credential should be set up for (payment | transfer).
        If not provided, defaults to payment.
        """

    class CreateParamsPayoutMethodData(TypedDict):
        type: Literal["bank_account", "card"]
        """
        Closed Enum. The type of payout method to be created.
        """
        bank_account: NotRequired[
            "OutboundSetupIntentService.CreateParamsPayoutMethodDataBankAccount"
        ]
        """
        The type specific details of the bank account payout method.
        """
        card: NotRequired[
            "OutboundSetupIntentService.CreateParamsPayoutMethodDataCard"
        ]
        """
        The type specific details of the card payout method.
        """

    class CreateParamsPayoutMethodDataBankAccount(TypedDict):
        account_number: str
        """
        The account number or IBAN of the bank account.
        """
        bank_account_type: NotRequired[Literal["checking", "savings"]]
        """
        Closed Enum. The type of the bank account (checking or savings).
        """
        branch_number: NotRequired[str]
        """
        The branch number of the bank account, if present.
        """
        country: str
        """
        The country code of the bank account.
        """
        routing_number: NotRequired[str]
        """
        The routing number of the bank account, if present.
        """
        swift_code: NotRequired[str]
        """
        The swift code of the bank account, if present.
        """

    class CreateParamsPayoutMethodDataCard(TypedDict):
        exp_month: str
        """
        The expiration month of the card.
        """
        exp_year: str
        """
        The expiration year of the card.
        """
        number: str
        """
        The card number.
        """

    class ListParams(TypedDict):
        limit: NotRequired[int]
        """
        The page size.
        """

    class RetrieveParams(TypedDict):
        pass

    class UpdateParams(TypedDict):
        payout_method: NotRequired[str]
        """
        If provided, the existing payout method resource to link to this outbound setup intent.
        """
        payout_method_data: NotRequired[
            "OutboundSetupIntentService.UpdateParamsPayoutMethodData"
        ]
        """
        If no payout_method provided, used to create the underlying credential that is set up for outbound money movement.
        If a payout_method provided, used to update data on the credential linked to this setup intent.
        """

    class UpdateParamsPayoutMethodData(TypedDict):
        type: Literal["bank_account", "card"]
        """
        Closed Enum. The type of payout method to be created/updated.
        """
        bank_account: NotRequired[
            "OutboundSetupIntentService.UpdateParamsPayoutMethodDataBankAccount"
        ]
        """
        The type specific details of the bank account payout method.
        """
        card: NotRequired[
            "OutboundSetupIntentService.UpdateParamsPayoutMethodDataCard"
        ]
        """
        The type specific details of the card payout method.
        """

    class UpdateParamsPayoutMethodDataBankAccount(TypedDict):
        account_number: str
        """
        The account number or IBAN of the bank account.
        """
        bank_account_type: NotRequired[Literal["checking", "savings"]]
        """
        Closed Enum. The type of the bank account (checking or savings).
        """
        branch_number: NotRequired[str]
        """
        The branch number of the bank account, if present.
        """
        country: str
        """
        The country code of the bank account.
        """
        routing_number: NotRequired[str]
        """
        The routing number of the bank account, if present.
        """
        swift_code: NotRequired[str]
        """
        The swift code of the bank account, if present.
        """

    class UpdateParamsPayoutMethodDataCard(TypedDict):
        exp_month: NotRequired[str]
        """
        The expiration month of the card.
        """
        exp_year: NotRequired[str]
        """
        The expiration year of the card.
        """
        number: NotRequired[str]
        """
        The card number. This can only be passed when creating a new credential on an outbound setup intent in the requires_payout_method state.
        """

    def cancel(
        self,
        id: str,
        params: "OutboundSetupIntentService.CancelParams" = {},
        options: RequestOptions = {},
    ) -> OutboundSetupIntent:
        """
        Cancel an OutboundSetupIntent object.
        """
        return cast(
            OutboundSetupIntent,
            self._request(
                "post",
                "/v2/money_management/outbound_setup_intents/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        id: str,
        params: "OutboundSetupIntentService.CancelParams" = {},
        options: RequestOptions = {},
    ) -> OutboundSetupIntent:
        """
        Cancel an OutboundSetupIntent object.
        """
        return cast(
            OutboundSetupIntent,
            await self._request_async(
                "post",
                "/v2/money_management/outbound_setup_intents/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "OutboundSetupIntentService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> OutboundSetupIntent:
        """
        Create an OutboundSetupIntent object.
        """
        return cast(
            OutboundSetupIntent,
            self._request(
                "post",
                "/v2/money_management/outbound_setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "OutboundSetupIntentService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> OutboundSetupIntent:
        """
        Create an OutboundSetupIntent object.
        """
        return cast(
            OutboundSetupIntent,
            await self._request_async(
                "post",
                "/v2/money_management/outbound_setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        params: "OutboundSetupIntentService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[OutboundSetupIntent]:
        """
        List the OutboundSetupIntent objects.
        """
        return cast(
            ListObject[OutboundSetupIntent],
            self._request(
                "get",
                "/v2/money_management/outbound_setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "OutboundSetupIntentService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[OutboundSetupIntent]:
        """
        List the OutboundSetupIntent objects.
        """
        return cast(
            ListObject[OutboundSetupIntent],
            await self._request_async(
                "get",
                "/v2/money_management/outbound_setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: "OutboundSetupIntentService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> OutboundSetupIntent:
        """
        Retrieve an OutboundSetupIntent object.
        """
        return cast(
            OutboundSetupIntent,
            self._request(
                "get",
                "/v2/money_management/outbound_setup_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: "OutboundSetupIntentService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> OutboundSetupIntent:
        """
        Retrieve an OutboundSetupIntent object.
        """
        return cast(
            OutboundSetupIntent,
            await self._request_async(
                "get",
                "/v2/money_management/outbound_setup_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: "OutboundSetupIntentService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> OutboundSetupIntent:
        """
        Update an OutboundSetupIntent object.
        """
        return cast(
            OutboundSetupIntent,
            self._request(
                "post",
                "/v2/money_management/outbound_setup_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: "OutboundSetupIntentService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> OutboundSetupIntent:
        """
        Update an OutboundSetupIntent object.
        """
        return cast(
            OutboundSetupIntent,
            await self._request_async(
                "post",
                "/v2/money_management/outbound_setup_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._request_options import RequestOptions
from stripe._stripe_object import StripeObject
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import (
    Literal,
    NotRequired,
    TypedDict,
    Unpack,
    TYPE_CHECKING,
)

if TYPE_CHECKING:
    from stripe.tax._calculation_line_item import CalculationLineItem


class Calculation(CreateableAPIResource["Calculation"]):
    """
    A Tax Calculation allows you to calculate the tax to collect from your customer.

    Related guide: [Calculate tax in your custom payment flow](https://stripe.com/docs/tax/custom)
    """

    OBJECT_NAME: ClassVar[Literal["tax.calculation"]] = "tax.calculation"

    class CustomerDetails(StripeObject):
        class Address(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: str
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1 (e.g., street, PO Box, or company name).
            """
            line2: Optional[str]
            """
            Address line 2 (e.g., apartment, suite, unit, or building).
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State/province as an [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code, without country prefix. Example: "NY" or "TX".
            """

        class TaxId(StripeObject):
            type: Literal[
                "ad_nrt",
                "ae_trn",
                "al_tin",
                "am_tin",
                "ao_tin",
                "ar_cuit",
                "au_abn",
                "au_arn",
                "ba_tin",
                "bb_tin",
                "bg_uic",
                "bh_vat",
                "bo_tin",
                "br_cnpj",
                "br_cpf",
                "bs_tin",
                "by_tin",
                "ca_bn",
                "ca_gst_hst",
                "ca_pst_bc",
                "ca_pst_mb",
                "ca_pst_sk",
                "ca_qst",
                "cd_nif",
                "ch_uid",
                "ch_vat",
                "cl_tin",
                "cn_tin",
                "co_nit",
                "cr_tin",
                "de_stn",
                "do_rcn",
                "ec_ruc",
                "eg_tin",
                "es_cif",
                "eu_oss_vat",
                "eu_vat",
                "gb_vat",
                "ge_vat",
                "gn_nif",
                "hk_br",
                "hr_oib",
                "hu_tin",
                "id_npwp",
                "il_vat",
                "in_gst",
                "is_vat",
                "jp_cn",
                "jp_rn",
                "jp_trn",
                "ke_pin",
                "kh_tin",
                "kr_brn",
                "kz_bin",
                "li_uid",
                "li_vat",
                "ma_vat",
                "md_vat",
                "me_pib",
                "mk_vat",
                "mr_nif",
                "mx_rfc",
                "my_frp",
                "my_itn",
                "my_sst",
                "ng_tin",
                "no_vat",
                "no_voec",
                "np_pan",
                "nz_gst",
                "om_vat",
                "pe_ruc",
                "ph_tin",
                "ro_tin",
                "rs_pib",
                "ru_inn",
                "ru_kpp",
                "sa_vat",
                "sg_gst",
                "sg_uen",
                "si_tin",
                "sn_ninea",
                "sr_fin",
                "sv_nit",
                "th_vat",
                "tj_tin",
                "tr_tin",
                "tw_vat",
                "tz_vat",
                "ua_vat",
                "ug_tin",
                "unknown",
                "us_ein",
                "uy_ruc",
                "uz_tin",
                "uz_vat",
                "ve_rif",
                "vn_tin",
                "za_vat",
                "zm_tin",
                "zw_tin",
            ]
            """
            The type of the tax ID, one of `ad_nrt`, `ar_cuit`, `eu_vat`, `bo_tin`, `br_cnpj`, `br_cpf`, `cn_tin`, `co_nit`, `cr_tin`, `do_rcn`, `ec_ruc`, `eu_oss_vat`, `hr_oib`, `pe_ruc`, `ro_tin`, `rs_pib`, `sv_nit`, `uy_ruc`, `ve_rif`, `vn_tin`, `gb_vat`, `nz_gst`, `au_abn`, `au_arn`, `in_gst`, `no_vat`, `no_voec`, `za_vat`, `ch_vat`, `mx_rfc`, `sg_uen`, `ru_inn`, `ru_kpp`, `ca_bn`, `hk_br`, `es_cif`, `tw_vat`, `th_vat`, `jp_cn`, `jp_rn`, `jp_trn`, `li_uid`, `li_vat`, `my_itn`, `us_ein`, `kr_brn`, `ca_qst`, `ca_gst_hst`, `ca_pst_bc`, `ca_pst_mb`, `ca_pst_sk`, `my_sst`, `sg_gst`, `ae_trn`, `cl_tin`, `sa_vat`, `id_npwp`, `my_frp`, `il_vat`, `ge_vat`, `ua_vat`, `is_vat`, `bg_uic`, `hu_tin`, `si_tin`, `ke_pin`, `tr_tin`, `eg_tin`, `ph_tin`, `al_tin`, `bh_vat`, `kz_bin`, `ng_tin`, `om_vat`, `de_stn`, `ch_uid`, `tz_vat`, `uz_vat`, `uz_tin`, `md_vat`, `ma_vat`, `by_tin`, `ao_tin`, `bs_tin`, `bb_tin`, `cd_nif`, `mr_nif`, `me_pib`, `zw_tin`, `ba_tin`, `gn_nif`, `mk_vat`, `sr_fin`, `sn_ninea`, `am_tin`, `np_pan`, `tj_tin`, `ug_tin`, `zm_tin`, `kh_tin`, or `unknown`
            """
            value: str
            """
            The value of the tax ID.
            """

        address: Optional[Address]
        """
        The customer's postal address (for example, home or business location).
        """
        address_source: Optional[Literal["billing", "shipping"]]
        """
        The type of customer address provided.
        """
        ip_address: Optional[str]
        """
        The customer's IP address (IPv4 or IPv6).
        """
        tax_ids: List[TaxId]
        """
        The customer's tax IDs (for example, EU VAT numbers).
        """
        taxability_override: Literal[
            "customer_exempt", "none", "reverse_charge"
        ]
        """
        The taxability override used for taxation.
        """
        _inner_class_types = {"address": Address, "tax_ids": TaxId}

    class ShipFromDetails(StripeObject):
        class Address(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: str
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1 (e.g., street, PO Box, or company name).
            """
            line2: Optional[str]
            """
            Address line 2 (e.g., apartment, suite, unit, or building).
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State/province as an [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code, without country prefix. Example: "NY" or "TX".
            """

        address: Address
        _inner_class_types = {"address": Address}

    class ShippingCost(StripeObject):
        class TaxBreakdown(StripeObject):
            class Jurisdiction(StripeObject):
                country: str
                """
                Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                """
                display_name: str
                """
                A human-readable name for the jurisdiction imposing the tax.
                """
                level: Literal[
                    "city", "country", "county", "district", "state"
                ]
                """
                Indicates the level of the jurisdiction imposing the tax.
                """
                state: Optional[str]
                """
                [ISO 3166-2 subdivision code](https://en.wikipedia.org/wiki/ISO_3166-2), without country prefix. For example, "NY" for New York, United States.
                """

            class TaxRateDetails(StripeObject):
                display_name: str
                """
                A localized display name for tax type, intended to be human-readable. For example, "Local Sales and Use Tax", "Value-added tax (VAT)", or "Umsatzsteuer (USt.)".
                """
                percentage_decimal: str
                """
                The tax rate percentage as a string. For example, 8.5% is represented as "8.5".
                """
                tax_type: Literal[
                    "amusement_tax",
                    "communications_tax",
                    "gst",
                    "hst",
                    "igst",
                    "jct",
                    "lease_tax",
                    "pst",
                    "qst",
                    "retail_delivery_fee",
                    "rst",
                    "sales_tax",
                    "service_tax",
                    "vat",
                ]
                """
                The tax type, such as `vat` or `sales_tax`.
                """

            amount: int
            """
            The amount of tax, in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
            """
            jurisdiction: Jurisdiction
            sourcing: Literal["destination", "origin"]
            """
            Indicates whether the jurisdiction was determined by the origin (merchant's address) or destination (customer's address).
            """
            tax_rate_details: Optional[TaxRateDetails]
            """
            Details regarding the rate for this tax. This field will be `null` when the tax is not imposed, for example if the product is exempt from tax.
            """
            taxability_reason: Literal[
                "customer_exempt",
                "not_collecting",
                "not_subject_to_tax",
                "not_supported",
                "portion_product_exempt",
                "portion_reduced_rated",
                "portion_standard_rated",
                "product_exempt",
                "product_exempt_holiday",
                "proportionally_rated",
                "reduced_rated",
                "reverse_charge",
                "standard_rated",
                "taxable_basis_reduced",
                "zero_rated",
            ]
            """
            The reasoning behind this tax, for example, if the product is tax exempt. The possible values for this field may be extended as new tax rules are supported.
            """
            taxable_amount: int
            """
            The amount on which tax is calculated, in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
            """
            _inner_class_types = {
                "jurisdiction": Jurisdiction,
                "tax_rate_details": TaxRateDetails,
            }

        amount: int
        """
        The shipping amount in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal). If `tax_behavior=inclusive`, then this amount includes taxes. Otherwise, taxes were calculated on top of this amount.
        """
        amount_tax: int
        """
        The amount of tax calculated for shipping, in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        shipping_rate: Optional[str]
        """
        The ID of an existing [ShippingRate](https://stripe.com/docs/api/shipping_rates/object).
        """
        tax_behavior: Literal["exclusive", "inclusive"]
        """
        Specifies whether the `amount` includes taxes. If `tax_behavior=inclusive`, then the amount includes taxes.
        """
        tax_breakdown: Optional[List[TaxBreakdown]]
        """
        Detailed account of taxes relevant to shipping cost.
        """
        tax_code: str
        """
        The [tax code](https://stripe.com/docs/tax/tax-categories) ID used for shipping.
        """
        _inner_class_types = {"tax_breakdown": TaxBreakdown}

    class TaxBreakdown(StripeObject):
        class TaxRateDetails(StripeObject):
            class FlatAmount(StripeObject):
                amount: int
                """
                Amount of the tax when the `rate_type` is `flat_amount`. This positive integer represents how much to charge in the smallest currency unit (e.g., 100 cents to charge $1.00 or 100 to charge ¥100, a zero-decimal currency). The amount value supports up to eight digits (e.g., a value of 99999999 for a USD charge of $999,999.99).
                """
                currency: str
                """
                Three-letter ISO currency code, in lowercase.
                """

            country: Optional[str]
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            flat_amount: Optional[FlatAmount]
            """
            The amount of the tax rate when the `rate_type` is `flat_amount`. Tax rates with `rate_type` `percentage` can vary based on the transaction, resulting in this field being `null`. This field exposes the amount and currency of the flat tax rate.
            """
            percentage_decimal: str
            """
            The tax rate percentage as a string. For example, 8.5% is represented as `"8.5"`.
            """
            rate_type: Optional[Literal["flat_amount", "percentage"]]
            """
            Indicates the type of tax rate applied to the taxable amount. This value can be `null` when no tax applies to the location. This field is only present for TaxRates created by Stripe Tax.
            """
            state: Optional[str]
            """
            State, county, province, or region.
            """
            tax_type: Optional[
                Literal[
                    "amusement_tax",
                    "communications_tax",
                    "gst",
                    "hst",
                    "igst",
                    "jct",
                    "lease_tax",
                    "pst",
                    "qst",
                    "retail_delivery_fee",
                    "rst",
                    "sales_tax",
                    "service_tax",
                    "vat",
                ]
            ]
            """
            The tax type, such as `vat` or `sales_tax`.
            """
            _inner_class_types = {"flat_amount": FlatAmount}

        amount: int
        """
        The amount of tax, in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        inclusive: bool
        """
        Specifies whether the tax amount is included in the line item amount.
        """
        tax_rate_details: TaxRateDetails
        taxability_reason: Literal[
            "customer_exempt",
            "not_collecting",
            "not_subject_to_tax",
            "not_supported",
            "portion_product_exempt",
            "portion_reduced_rated",
            "portion_standard_rated",
            "product_exempt",
            "product_exempt_holiday",
            "proportionally_rated",
            "reduced_rated",
            "reverse_charge",
            "standard_rated",
            "taxable_basis_reduced",
            "zero_rated",
        ]
        """
        The reasoning behind this tax, for example, if the product is tax exempt. We might extend the possible values for this field to support new tax rules.
        """
        taxable_amount: int
        """
        The amount on which tax is calculated, in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        _inner_class_types = {"tax_rate_details": TaxRateDetails}

    class CreateParams(RequestOptions):
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        customer: NotRequired[str]
        """
        The ID of an existing customer to use for this calculation. If provided, the customer's address and tax IDs are copied to `customer_details`.
        """
        customer_details: NotRequired[
            "Calculation.CreateParamsCustomerDetails"
        ]
        """
        Details about the customer, including address and tax IDs.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        line_items: List["Calculation.CreateParamsLineItem"]
        """
        A list of items the customer is purchasing.
        """
        ship_from_details: NotRequired[
            "Calculation.CreateParamsShipFromDetails"
        ]
        """
        Details about the address from which the goods are being shipped.
        """
        shipping_cost: NotRequired["Calculation.CreateParamsShippingCost"]
        """
        Shipping cost details to be used for the calculation.
        """
        tax_date: NotRequired[int]
        """
        Timestamp of date at which the tax rules and rates in effect applies for the calculation. Measured in seconds since the Unix epoch. Can be up to 48 hours in the past, and up to 48 hours in the future.
        """

    class CreateParamsCustomerDetails(TypedDict):
        address: NotRequired["Calculation.CreateParamsCustomerDetailsAddress"]
        """
        The customer's postal address (for example, home or business location).
        """
        address_source: NotRequired[Literal["billing", "shipping"]]
        """
        The type of customer address provided.
        """
        ip_address: NotRequired[str]
        """
        The customer's IP address (IPv4 or IPv6).
        """
        tax_ids: NotRequired[
            List["Calculation.CreateParamsCustomerDetailsTaxId"]
        ]
        """
        The customer's tax IDs. Stripe Tax might consider a transaction with applicable tax IDs to be B2B, which might affect the tax calculation result. Stripe Tax doesn't validate tax IDs for correctness.
        """
        taxability_override: NotRequired[
            Literal["customer_exempt", "none", "reverse_charge"]
        ]
        """
        Overrides the tax calculation result to allow you to not collect tax from your customer. Use this if you've manually checked your customer's tax exemptions. Prefer providing the customer's `tax_ids` where possible, which automatically determines whether `reverse_charge` applies.
        """

    class CreateParamsCustomerDetailsAddress(TypedDict):
        city: NotRequired["Literal['']|str"]
        """
        City, district, suburb, town, or village.
        """
        country: str
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["Literal['']|str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["Literal['']|str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["Literal['']|str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["Literal['']|str"]
        """
        State, county, province, or region. We recommend sending [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code value when possible.
        """

    class CreateParamsCustomerDetailsTaxId(TypedDict):
        type: Literal[
            "ad_nrt",
            "ae_trn",
            "al_tin",
            "am_tin",
            "ao_tin",
            "ar_cuit",
            "au_abn",
            "au_arn",
            "ba_tin",
            "bb_tin",
            "bg_uic",
            "bh_vat",
            "bo_tin",
            "br_cnpj",
            "br_cpf",
            "bs_tin",
            "by_tin",
            "ca_bn",
            "ca_gst_hst",
            "ca_pst_bc",
            "ca_pst_mb",
            "ca_pst_sk",
            "ca_qst",
            "cd_nif",
            "ch_uid",
            "ch_vat",
            "cl_tin",
            "cn_tin",
            "co_nit",
            "cr_tin",
            "de_stn",
            "do_rcn",
            "ec_ruc",
            "eg_tin",
            "es_cif",
            "eu_oss_vat",
            "eu_vat",
            "gb_vat",
            "ge_vat",
            "gn_nif",
            "hk_br",
            "hr_oib",
            "hu_tin",
            "id_npwp",
            "il_vat",
            "in_gst",
            "is_vat",
            "jp_cn",
            "jp_rn",
            "jp_trn",
            "ke_pin",
            "kh_tin",
            "kr_brn",
            "kz_bin",
            "li_uid",
            "li_vat",
            "ma_vat",
            "md_vat",
            "me_pib",
            "mk_vat",
            "mr_nif",
            "mx_rfc",
            "my_frp",
            "my_itn",
            "my_sst",
            "ng_tin",
            "no_vat",
            "no_voec",
            "np_pan",
            "nz_gst",
            "om_vat",
            "pe_ruc",
            "ph_tin",
            "ro_tin",
            "rs_pib",
            "ru_inn",
            "ru_kpp",
            "sa_vat",
            "sg_gst",
            "sg_uen",
            "si_tin",
            "sn_ninea",
            "sr_fin",
            "sv_nit",
            "th_vat",
            "tj_tin",
            "tr_tin",
            "tw_vat",
            "tz_vat",
            "ua_vat",
            "ug_tin",
            "us_ein",
            "uy_ruc",
            "uz_tin",
            "uz_vat",
            "ve_rif",
            "vn_tin",
            "za_vat",
            "zm_tin",
            "zw_tin",
        ]
        """
        Type of the tax ID, one of `ad_nrt`, `ae_trn`, `al_tin`, `am_tin`, `ao_tin`, `ar_cuit`, `au_abn`, `au_arn`, `ba_tin`, `bb_tin`, `bg_uic`, `bh_vat`, `bo_tin`, `br_cnpj`, `br_cpf`, `bs_tin`, `by_tin`, `ca_bn`, `ca_gst_hst`, `ca_pst_bc`, `ca_pst_mb`, `ca_pst_sk`, `ca_qst`, `cd_nif`, `ch_uid`, `ch_vat`, `cl_tin`, `cn_tin`, `co_nit`, `cr_tin`, `de_stn`, `do_rcn`, `ec_ruc`, `eg_tin`, `es_cif`, `eu_oss_vat`, `eu_vat`, `gb_vat`, `ge_vat`, `gn_nif`, `hk_br`, `hr_oib`, `hu_tin`, `id_npwp`, `il_vat`, `in_gst`, `is_vat`, `jp_cn`, `jp_rn`, `jp_trn`, `ke_pin`, `kh_tin`, `kr_brn`, `kz_bin`, `li_uid`, `li_vat`, `ma_vat`, `md_vat`, `me_pib`, `mk_vat`, `mr_nif`, `mx_rfc`, `my_frp`, `my_itn`, `my_sst`, `ng_tin`, `no_vat`, `no_voec`, `np_pan`, `nz_gst`, `om_vat`, `pe_ruc`, `ph_tin`, `ro_tin`, `rs_pib`, `ru_inn`, `ru_kpp`, `sa_vat`, `sg_gst`, `sg_uen`, `si_tin`, `sn_ninea`, `sr_fin`, `sv_nit`, `th_vat`, `tj_tin`, `tr_tin`, `tw_vat`, `tz_vat`, `ua_vat`, `ug_tin`, `us_ein`, `uy_ruc`, `uz_tin`, `uz_vat`, `ve_rif`, `vn_tin`, `za_vat`, `zm_tin`, or `zw_tin`
        """
        value: str
        """
        Value of the tax ID.
        """

    class CreateParamsLineItem(TypedDict):
        amount: int
        """
        A positive integer representing the line item's total price in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        If `tax_behavior=inclusive`, then this amount includes taxes. Otherwise, taxes are calculated on top of this amount.
        """
        product: NotRequired[str]
        """
        If provided, the product's `tax_code` will be used as the line item's `tax_code`.
        """
        quantity: NotRequired[int]
        """
        The number of units of the item being purchased. Used to calculate the per-unit price from the total `amount` for the line. For example, if `amount=100` and `quantity=4`, the calculated unit price is 25.
        """
        reference: NotRequired[str]
        """
        A custom identifier for this line item, which must be unique across the line items in the calculation. The reference helps identify each line item in exported [tax reports](https://stripe.com/docs/tax/reports).
        """
        tax_behavior: NotRequired[Literal["exclusive", "inclusive"]]
        """
        Specifies whether the `amount` includes taxes. Defaults to `exclusive`.
        """
        tax_code: NotRequired[str]
        """
        A [tax code](https://stripe.com/docs/tax/tax-categories) ID to use for this line item. If not provided, we will use the tax code from the provided `product` param. If neither `tax_code` nor `product` is provided, we will use the default tax code from your Tax Settings.
        """

    class CreateParamsShipFromDetails(TypedDict):
        address: "Calculation.CreateParamsShipFromDetailsAddress"
        """
        The address from which the goods are being shipped from.
        """

    class CreateParamsShipFromDetailsAddress(TypedDict):
        city: NotRequired["Literal['']|str"]
        """
        City, district, suburb, town, or village.
        """
        country: str
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["Literal['']|str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["Literal['']|str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["Literal['']|str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["Literal['']|str"]
        """
        State/province as an [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code, without country prefix. Example: "NY" or "TX".
        """

    class CreateParamsShippingCost(TypedDict):
        amount: NotRequired[int]
        """
        A positive integer in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal) representing the shipping charge. If `tax_behavior=inclusive`, then this amount includes taxes. Otherwise, taxes are calculated on top of this amount.
        """
        shipping_rate: NotRequired[str]
        """
        If provided, the [shipping rate](https://stripe.com/docs/api/shipping_rates/object)'s `amount`, `tax_code` and `tax_behavior` are used. If you provide a shipping rate, then you cannot pass the `amount`, `tax_code`, or `tax_behavior` parameters.
        """
        tax_behavior: NotRequired[Literal["exclusive", "inclusive"]]
        """
        Specifies whether the `amount` includes taxes. If `tax_behavior=inclusive`, then the amount includes taxes. Defaults to `exclusive`.
        """
        tax_code: NotRequired[str]
        """
        The [tax code](https://stripe.com/docs/tax/tax-categories) used to calculate tax on shipping. If not provided, the default shipping tax code from your [Tax Settings](https://dashboard.stripe.com/settings/tax) is used.
        """

    class ListLineItemsParams(RequestOptions):
        ending_before: NotRequired[str]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired[int]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        starting_after: NotRequired[str]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """

    class RetrieveParams(RequestOptions):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    amount_total: int
    """
    Total amount after taxes in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    customer: Optional[str]
    """
    The ID of an existing [Customer](https://stripe.com/docs/api/customers/object) used for the resource.
    """
    customer_details: CustomerDetails
    expires_at: Optional[int]
    """
    Timestamp of date at which the tax calculation will expire.
    """
    id: Optional[str]
    """
    Unique identifier for the calculation.
    """
    line_items: Optional[ListObject["CalculationLineItem"]]
    """
    The list of items the customer is purchasing.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["tax.calculation"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    ship_from_details: Optional[ShipFromDetails]
    """
    The details of the ship from location, such as the address.
    """
    shipping_cost: Optional[ShippingCost]
    """
    The shipping cost details for the calculation.
    """
    tax_amount_exclusive: int
    """
    The amount of tax to be collected on top of the line item prices.
    """
    tax_amount_inclusive: int
    """
    The amount of tax already included in the line item prices.
    """
    tax_breakdown: List[TaxBreakdown]
    """
    Breakdown of individual tax amounts that add up to the total.
    """
    tax_date: int
    """
    Timestamp of date at which the tax rules and rates in effect applies for the calculation.
    """

    @classmethod
    def create(
        cls, **params: Unpack["Calculation.CreateParams"]
    ) -> "Calculation":
        """
        Calculates tax based on the input and returns a Tax Calculation object.
        """
        return cast(
            "Calculation",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["Calculation.CreateParams"]
    ) -> "Calculation":
        """
        Calculates tax based on the input and returns a Tax Calculation object.
        """
        return cast(
            "Calculation",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_list_line_items(
        cls,
        calculation: str,
        **params: Unpack["Calculation.ListLineItemsParams"],
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            ListObject["CalculationLineItem"],
            cls._static_request(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(calculation)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def list_line_items(
        calculation: str, **params: Unpack["Calculation.ListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        ...

    @overload
    def list_line_items(
        self, **params: Unpack["Calculation.ListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        ...

    @class_method_variant("_cls_list_line_items")
    def list_line_items(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["Calculation.ListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            ListObject["CalculationLineItem"],
            self._request(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_list_line_items_async(
        cls,
        calculation: str,
        **params: Unpack["Calculation.ListLineItemsParams"],
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            ListObject["CalculationLineItem"],
            await cls._static_request_async(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(calculation)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def list_line_items_async(
        calculation: str, **params: Unpack["Calculation.ListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        ...

    @overload
    async def list_line_items_async(
        self, **params: Unpack["Calculation.ListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        ...

    @class_method_variant("_cls_list_line_items_async")
    async def list_line_items_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["Calculation.ListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            ListObject["CalculationLineItem"],
            await self._request_async(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["Calculation.RetrieveParams"]
    ) -> "Calculation":
        """
        Retrieves a Tax Calculation object, if the calculation hasn't expired.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["Calculation.RetrieveParams"]
    ) -> "Calculation":
        """
        Retrieves a Tax Calculation object, if the calculation hasn't expired.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "customer_details": CustomerDetails,
        "ship_from_details": ShipFromDetails,
        "shipping_cost": ShippingCost,
        "tax_breakdown": TaxBreakdown,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, Optional
from typing_extensions import Literal


class Adjustment(StripeObject):
    """
    Adjustments represent Stripe-initiated credits or debits to a user balance. They might be used to amend balances due to technical or operational error.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.adjustment"]] = (
        "v2.money_management.adjustment"
    )

    class AdjustedFlow(StripeObject):
        type: Literal[
            "adjustment",
            "balance_exchange",
            "inbound_payment",
            "inbound_transfer",
            "outbound_payment",
            "outbound_transfer",
            "received_credit",
            "received_debit",
        ]
        """
        Closed Enum. If applicable, the type of flow linked to this Adjustment. The field matching this value will contain the ID of the flow.
        """
        adjustment: Optional[str]
        """
        If applicable, the ID of the Adjustment linked to this Adjustment.
        """
        inbound_transfer: Optional[str]
        """
        If applicable, the ID of the InboundTransfer linked to this Adjustment.
        """
        outbound_payment: Optional[str]
        """
        If applicable, the ID of the OutboundPayment linked to this Adjustment.
        """
        outbound_transfer: Optional[str]
        """
        If applicable, the ID of the OutboundTransfer linked to this Adjustment.
        """
        received_credit: Optional[str]
        """
        If applicable, the ID of the ReceivedCredit linked to this Adjustment.
        """
        received_debit: Optional[str]
        """
        If applicable, the ID of the ReceivedDebit linked to this Adjustment.
        """

    adjusted_flow: Optional[AdjustedFlow]
    """
    If applicable, contains information about the original flow linked to this Adjustment.
    """
    amount: Amount
    """
    The amount of the Adjustment.
    """
    created: str
    """
    Time at which the object was created. Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    description: Optional[str]
    """
    Description of the Adjustment and what it was used for.
    """
    financial_account: str
    """
    The FinancialAccount that this adjustment is for.
    """
    id: str
    """
    Unique identifier for the object.
    """
    object: Literal["v2.money_management.adjustment"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    receipt_url: str
    """
    A hosted transaction receipt URL that is provided when money movement is considered regulated under Stripe's money transmission licenses.
    """
    _inner_class_types = {"adjusted_flow": AdjustedFlow}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe.v2.billing._meter_event_adjustment_service import (
    MeterEventAdjustmentService,
)
from stripe.v2.billing._meter_event_service import MeterEventService
from stripe.v2.billing._meter_event_session_service import (
    MeterEventSessionService,
)
from stripe.v2.billing._meter_event_stream_service import (
    MeterEventStreamService,
)


class BillingService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.meter_event_session = MeterEventSessionService(self._requestor)
        self.meter_event_adjustments = MeterEventAdjustmentService(
            self._requestor,
        )
        self.meter_event_stream = MeterEventStreamService(self._requestor)
        self.meter_events = MeterEventService(self._requestor)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.issuing._authorization import Authorization
from typing import Dict, List, cast
from typing_extensions import Literal, NotRequired, TypedDict


class AuthorizationService(StripeService):
    class ApproveParams(TypedDict):
        amount: NotRequired[int]
        """
        If the authorization's `pending_request.is_amount_controllable` property is `true`, you may provide this value to control how much to hold for the authorization. Must be positive (use [`decline`](https://stripe.com/docs/api/issuing/authorizations/decline) to decline an authorization request).
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """

    class DeclineParams(TypedDict):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """

    class ListParams(TypedDict):
        card: NotRequired[str]
        """
        Only return authorizations that belong to the given card.
        """
        cardholder: NotRequired[str]
        """
        Only return authorizations that belong to the given cardholder.
        """
        created: NotRequired["AuthorizationService.ListParamsCreated|int"]
        """
        Only return authorizations that were created during the given date interval.
        """
        ending_before: NotRequired[str]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired[int]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        starting_after: NotRequired[str]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """
        status: NotRequired[
            Literal["closed", "expired", "pending", "reversed"]
        ]
        """
        Only return authorizations with the given status. One of `pending`, `closed`, or `reversed`.
        """

    class ListParamsCreated(TypedDict):
        gt: NotRequired[int]
        """
        Minimum value to filter by (exclusive)
        """
        gte: NotRequired[int]
        """
        Minimum value to filter by (inclusive)
        """
        lt: NotRequired[int]
        """
        Maximum value to filter by (exclusive)
        """
        lte: NotRequired[int]
        """
        Maximum value to filter by (inclusive)
        """

    class RetrieveParams(TypedDict):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    class UpdateParams(TypedDict):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """

    def list(
        self,
        params: "AuthorizationService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Authorization]:
        """
        Returns a list of Issuing Authorization objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        return cast(
            ListObject[Authorization],
            self._request(
                "get",
                "/v1/issuing/authorizations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "AuthorizationService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Authorization]:
        """
        Returns a list of Issuing Authorization objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        return cast(
            ListObject[Authorization],
            await self._request_async(
                "get",
                "/v1/issuing/authorizations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        authorization: str,
        params: "AuthorizationService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Retrieves an Issuing Authorization object.
        """
        return cast(
            Authorization,
            self._request(
                "get",
                "/v1/issuing/authorizations/{authorization}".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        authorization: str,
        params: "AuthorizationService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Retrieves an Issuing Authorization object.
        """
        return cast(
            Authorization,
            await self._request_async(
                "get",
                "/v1/issuing/authorizations/{authorization}".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        authorization: str,
        params: "AuthorizationService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Updates the specified Issuing Authorization object by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        return cast(
            Authorization,
            self._request(
                "post",
                "/v1/issuing/authorizations/{authorization}".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        authorization: str,
        params: "AuthorizationService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Updates the specified Issuing Authorization object by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        return cast(
            Authorization,
            await self._request_async(
                "post",
                "/v1/issuing/authorizations/{authorization}".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def approve(
        self,
        authorization: str,
        params: "AuthorizationService.ApproveParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        [Deprecated] Approves a pending Issuing Authorization object. This request should be made within the timeout window of the [real-time authorization](https://stripe.com/docs/issuing/controls/real-time-authorizations) flow.
        This method is deprecated. Instead, [respond directly to the webhook request to approve an authorization](https://stripe.com/docs/issuing/controls/real-time-authorizations#authorization-handling).
        """
        return cast(
            Authorization,
            self._request(
                "post",
                "/v1/issuing/authorizations/{authorization}/approve".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def approve_async(
        self,
        authorization: str,
        params: "AuthorizationService.ApproveParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        [Deprecated] Approves a pending Issuing Authorization object. This request should be made within the timeout window of the [real-time authorization](https://stripe.com/docs/issuing/controls/real-time-authorizations) flow.
        This method is deprecated. Instead, [respond directly to the webhook request to approve an authorization](https://stripe.com/docs/issuing/controls/real-time-authorizations#authorization-handling).
        """
        return cast(
            Authorization,
            await self._request_async(
                "post",
                "/v1/issuing/authorizations/{authorization}/approve".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def decline(
        self,
        authorization: str,
        params: "AuthorizationService.DeclineParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        [Deprecated] Declines a pending Issuing Authorization object. This request should be made within the timeout window of the [real time authorization](https://stripe.com/docs/issuing/controls/real-time-authorizations) flow.
        This method is deprecated. Instead, [respond directly to the webhook request to decline an authorization](https://stripe.com/docs/issuing/controls/real-time-authorizations#authorization-handling).
        """
        return cast(
            Authorization,
            self._request(
                "post",
                "/v1/issuing/authorizations/{authorization}/decline".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def decline_async(
        self,
        authorization: str,
        params: "AuthorizationService.DeclineParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        [Deprecated] Declines a pending Issuing Authorization object. This request should be made within the timeout window of the [real time authorization](https://stripe.com/docs/issuing/controls/real-time-authorizations) flow.
        This method is deprecated. Instead, [respond directly to the webhook request to decline an authorization](https://stripe.com/docs/issuing/controls/real-time-authorizations#authorization-handling).
        """
        return cast(
            Authorization,
            await self._request_async(
                "post",
                "/v1/issuing/authorizations/{authorization}/decline".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2._list_object import ListObject
from stripe.v2.money_management._adjustment import Adjustment
from typing import cast
from typing_extensions import NotRequired, TypedDict


class AdjustmentService(StripeService):
    class ListParams(TypedDict):
        adjusted_flow: NotRequired[str]
        """
        Filter for Adjustments linked to a Flow.
        """
        created: NotRequired[str]
        """
        Filter for objects created at the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        created_gt: NotRequired[str]
        """
        Filter for objects created after the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        created_gte: NotRequired[str]
        """
        Filter for objects created on or after the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        created_lt: NotRequired[str]
        """
        Filter for objects created before the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        created_lte: NotRequired[str]
        """
        Filter for objects created on or before the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        limit: NotRequired[int]
        """
        The page limit.
        """

    class RetrieveParams(TypedDict):
        pass

    def list(
        self,
        params: "AdjustmentService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Adjustment]:
        """
        Returns a list of Adjustments that match the provided filters.
        """
        return cast(
            ListObject[Adjustment],
            self._request(
                "get",
                "/v2/money_management/adjustments",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "AdjustmentService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Adjustment]:
        """
        Returns a list of Adjustments that match the provided filters.
        """
        return cast(
            ListObject[Adjustment],
            await self._request_async(
                "get",
                "/v2/money_management/adjustments",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: "AdjustmentService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Adjustment:
        """
        Retrieves the details of an Adjustment by ID.
        """
        return cast(
            Adjustment,
            self._request(
                "get",
                "/v2/money_management/adjustments/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: "AdjustmentService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Adjustment:
        """
        Retrieves the details of an Adjustment by ID.
        """
        return cast(
            Adjustment,
            await self._request_async(
                "get",
                "/v2/money_management/adjustments/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



import pytest

import stripe


class TestEphemeralKey(object):
    def test_is_creatable(self, http_client_mock):
        resource = stripe.EphemeralKey.create(
            customer="cus_123", stripe_version="2017-05-25"
        )
        http_client_mock.assert_requested(
            "post",
            path="/v1/ephemeral_keys",
            stripe_version="2017-05-25",
            post_data="customer=cus_123",
        )
        assert isinstance(resource, stripe.EphemeralKey)

    def test_is_not_creatable_without_an_explicit_api_version(self):
        with pytest.raises(
            ValueError, match="stripe_version must be specified"
        ):
            stripe.EphemeralKey.create(customer="cus_123")

    def test_is_deletable(self, http_client_mock):
        resource = stripe.EphemeralKey.create(
            customer="cus_123", stripe_version="2017-05-25"
        )
        resource.delete()
        http_client_mock.assert_requested(
            "delete", path="/v1/ephemeral_keys/%s" % resource.id
        )
        assert isinstance(resource, stripe.EphemeralKey)

    def test_can_delete(self, http_client_mock):
        resource = stripe.EphemeralKey.delete("ephkey_123")
        http_client_mock.assert_requested(
            "delete", path="/v1/ephemeral_keys/ephkey_123"
        )
        assert isinstance(resource, stripe.EphemeralKey)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2._event import Event
from stripe.v2._list_object import ListObject
from typing import cast
from typing_extensions import NotRequired, TypedDict


class EventService(StripeService):
    class ListParams(TypedDict):
        limit: NotRequired[int]
        """
        The page size.
        """
        object_id: str
        """
        Primary object ID used to retrieve related events.
        """

    class RetrieveParams(TypedDict):
        pass

    def list(
        self, params: "EventService.ListParams", options: RequestOptions = {}
    ) -> ListObject[Event]:
        """
        List events, going back up to 30 days.
        """
        return cast(
            ListObject[Event],
            self._request(
                "get",
                "/v2/core/events",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self, params: "EventService.ListParams", options: RequestOptions = {}
    ) -> ListObject[Event]:
        """
        List events, going back up to 30 days.
        """
        return cast(
            ListObject[Event],
            await self._request_async(
                "get",
                "/v2/core/events",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: "EventService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Event:
        """
        Retrieves the details of an event.
        """
        return cast(
            Event,
            self._request(
                "get",
                "/v2/core/events/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: "EventService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Event:
        """
        Retrieves the details of an event.
        """
        return cast(
            Event,
            await self._request_async(
                "get",
                "/v2/core/events/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe.v2.billing._meter_event import MeterEvent
from typing import Dict, cast
from typing_extensions import NotRequired, TypedDict


class MeterEventService(StripeService):
    class CreateParams(TypedDict):
        event_name: str
        """
        The name of the meter event. Corresponds with the `event_name` field on a meter.
        """
        identifier: NotRequired[str]
        """
        A unique identifier for the event. If not provided, one will be generated.
        We recommend using a globally unique identifier for this. We'll enforce
        uniqueness within a rolling 24 hour period.
        """
        payload: Dict[str, str]
        """
        The payload of the event. This must contain the fields corresponding to a meter's
        `customer_mapping.event_payload_key` (default is `stripe_customer_id`) and
        `value_settings.event_payload_key` (default is `value`). Read more about
        the
        [payload](https://docs.stripe.com/billing/subscriptions/usage-based/recording-usage#payload-key-overrides).
        """
        timestamp: NotRequired[str]
        """
        The time of the event. Must be within the past 35 calendar days or up to
        5 minutes in the future. Defaults to current timestamp if not specified.
        """

    def create(
        self,
        params: "MeterEventService.CreateParams",
        options: RequestOptions = {},
    ) -> MeterEvent:
        """
        Creates a meter event. Events are validated synchronously, but are processed asynchronously. Supports up to 1,000 events per second in livemode. For higher rate-limits, please use meter event streams instead.
        """
        return cast(
            MeterEvent,
            self._request(
                "post",
                "/v2/billing/meter_events",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "MeterEventService.CreateParams",
        options: RequestOptions = {},
    ) -> MeterEvent:
        """
        Creates a meter event. Events are validated synchronously, but are processed asynchronously. Supports up to 1,000 events per second in livemode. For higher rate-limits, please use meter event streams instead.
        """
        return cast(
            MeterEvent,
            await self._request_async(
                "post",
                "/v2/billing/meter_events",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
from typing_extensions import TYPE_CHECKING
from warnings import warn

warn(
    """
    The stripe.api_version package is deprecated and will become internal in the future.
    """,
    DeprecationWarning,
)

if not TYPE_CHECKING:
    from stripe._api_version import (  # noqa
        _ApiVersion,
    )



from stripe.v2._list_object import ListObject as ListObject
from stripe.v2._amount import Amount as Amount, AmountParam as AmountParam


# The beginning of the section generated from our OpenAPI spec
from stripe.v2 import (
    billing as billing,
    core as core,
    money_management as money_management,
    test_helpers as test_helpers,
)
from stripe.v2._billing_service import BillingService as BillingService
from stripe.v2._core_service import CoreService as CoreService
from stripe.v2._event import Event as Event
from stripe.v2._event_destination import EventDestination as EventDestination
from stripe.v2._financial_address_credit_simulation import (
    FinancialAddressCreditSimulation as FinancialAddressCreditSimulation,
)
from stripe.v2._financial_address_generated_microdeposits import (
    FinancialAddressGeneratedMicrodeposits as FinancialAddressGeneratedMicrodeposits,
)
from stripe.v2._money_management_service import (
    MoneyManagementService as MoneyManagementService,
)
from stripe.v2._test_helper_service import (
    TestHelperService as TestHelperService,
)
# The end of the section generated from our OpenAPI spec



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._api_mode import ApiMode
from stripe._api_requestor import _APIRequestor
from stripe._stripe_object import StripeObject
from stripe._stripe_response import StripeResponse
from stripe.v2._event import Event
from stripe.v2.core._account import Account
from typing import Any, Dict, Optional, cast
from typing_extensions import Literal


class V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEvent(
    Event,
):
    LOOKUP_TYPE = (
        "v2.core.account[configuration.merchant].capability_status_updated"
    )
    type: Literal[
        "v2.core.account[configuration.merchant].capability_status_updated"
    ]

    class V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEventData(
        StripeObject,
    ):
        updated_capability: Literal[
            "ach_debit_payments",
            "acss_debit_payments",
            "affirm_payments",
            "afterpay_clearpay_payments",
            "alma_payments",
            "amazon_pay_payments",
            "au_becs_debit_payments",
            "bacs_debit_payments",
            "bancontact_payments",
            "blik_payments",
            "boleto_payments",
            "card_payments",
            "cartes_bancaires_payments",
            "cashapp_payments",
            "eps_payments",
            "fpx_payments",
            "gb_bank_transfer_payments",
            "grabpay_payments",
            "ideal_payments",
            "jcb_payments",
            "jp_bank_transfer_payments",
            "kakao_pay_payments",
            "klarna_payments",
            "konbini_payments",
            "kr_card_payments",
            "link_payments",
            "mobilepay_payments",
            "multibanco_payments",
            "mx_bank_transfer_payments",
            "naver_pay_payments",
            "oxxo_payments",
            "p24_payments",
            "payco_payments",
            "paynow_payments",
            "pay_by_bank_payments",
            "promptpay_payments",
            "revolut_pay_payments",
            "samsung_pay_payments",
            "sepa_bank_transfer_payments",
            "sepa_debit_payments",
            "swish_payments",
            "twint_payments",
            "us_bank_transfer_payments",
            "zip_payments",
        ]
        """
        Open Enum. The capability which had its status updated.
        """

    data: V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEventData
    """
    Data for the v2.core.account[configuration.merchant].capability_status_updated event
    """

    @classmethod
    def _construct_from(
        cls,
        *,
        values: Dict[str, Any],
        last_response: Optional[StripeResponse] = None,
        requestor: "_APIRequestor",
        api_mode: ApiMode,
    ) -> "V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEvent":
        evt = super()._construct_from(
            values=values,
            last_response=last_response,
            requestor=requestor,
            api_mode=api_mode,
        )
        if hasattr(evt, "data"):
            evt.data = V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEvent.V2CoreAccountIncludingConfigurationMerchantCapabilityStatusUpdatedEventData._construct_from(
                values=evt.data,
                last_response=last_response,
                requestor=requestor,
                api_mode=api_mode,
            )
        return evt

    class RelatedObject(StripeObject):
        id: str
        """
        Unique identifier for the object relevant to the event.
        """
        type: str
        """
        Type of the object relevant to the event.
        """
        url: str
        """
        URL to retrieve the resource.
        """

    related_object: RelatedObject
    """
    Object containing the reference to API resource relevant to the event
    """

    def fetch_related_object(self) -> Account:
        """
        Retrieves the related object from the API. Makes an API request on every call.
        """
        return cast(
            Account,
            self._requestor.request(
                "get",
                self.related_object.url,
                base_address="api",
                options={"stripe_account": self.context},
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2.core.vault._gb_bank_account import GbBankAccount
from typing import cast
from typing_extensions import Literal, NotRequired, TypedDict


class GbBankAccountService(StripeService):
    class AcknowledgeConfirmationOfPayeeParams(TypedDict):
        pass

    class ArchiveParams(TypedDict):
        pass

    class CreateParams(TypedDict):
        account_number: str
        """
        The Account Number of the bank account.
        """
        bank_account_type: NotRequired[Literal["checking", "savings"]]
        """
        Closed Enum. The type of the bank account (checking or savings).
        """
        confirmation_of_payee: NotRequired[
            "GbBankAccountService.CreateParamsConfirmationOfPayee"
        ]
        """
        Whether or not to automatically perform Confirmation of Payee to verify the users information
        against what was provided by the bank. Doing so is required for all bank accounts not owned
        by you before making domestic UK OutboundPayments.
        """
        sort_code: str
        """
        The Sort Code of the bank account.
        """

    class CreateParamsConfirmationOfPayee(TypedDict):
        business_type: NotRequired[Literal["business", "personal"]]
        """
        The business type to be checked against. Legal entity information will be used if unspecified.
        Closed enum.
        """
        initiate: bool
        """
        User specifies whether Confirmation of Payee is automatically initiated when creating the bank account.
        """
        name: NotRequired[str]
        """
        The name to be checked against. Legal entity information will be used if unspecified.
        """

    class InitiateConfirmationOfPayeeParams(TypedDict):
        business_type: NotRequired[Literal["business", "personal"]]
        """
        The business type to be checked against. Legal entity information will be used if unspecified.
        """
        name: NotRequired[str]
        """
        The name of the user to be checked against. Legal entity information will be used if unspecified.
        """

    class RetrieveParams(TypedDict):
        pass

    def acknowledge_confirmation_of_payee(
        self,
        id: str,
        params: "GbBankAccountService.AcknowledgeConfirmationOfPayeeParams" = {},
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Confirm that you have received the result of the Confirmation of Payee request, and that you are okay with
        proceeding to pay out to this bank account despite the account not matching, partially matching, or the service
        being unavailable. Once you confirm this, you will be able to send OutboundPayments, but this may lead to
        funds being sent to the wrong account, which we might not be able to recover.
        """
        return cast(
            GbBankAccount,
            self._request(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/acknowledge_confirmation_of_payee".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def acknowledge_confirmation_of_payee_async(
        self,
        id: str,
        params: "GbBankAccountService.AcknowledgeConfirmationOfPayeeParams" = {},
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Confirm that you have received the result of the Confirmation of Payee request, and that you are okay with
        proceeding to pay out to this bank account despite the account not matching, partially matching, or the service
        being unavailable. Once you confirm this, you will be able to send OutboundPayments, but this may lead to
        funds being sent to the wrong account, which we might not be able to recover.
        """
        return cast(
            GbBankAccount,
            await self._request_async(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/acknowledge_confirmation_of_payee".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def archive(
        self,
        id: str,
        params: "GbBankAccountService.ArchiveParams" = {},
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Archive a GbBankAccount object. Archived GbBankAccount objects cannot be used as outbound destinations
        and will not appear in the outbound destination list.
        """
        return cast(
            GbBankAccount,
            self._request(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/archive".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def archive_async(
        self,
        id: str,
        params: "GbBankAccountService.ArchiveParams" = {},
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Archive a GbBankAccount object. Archived GbBankAccount objects cannot be used as outbound destinations
        and will not appear in the outbound destination list.
        """
        return cast(
            GbBankAccount,
            await self._request_async(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/archive".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "GbBankAccountService.CreateParams",
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Create a GB bank account.
        """
        return cast(
            GbBankAccount,
            self._request(
                "post",
                "/v2/core/vault/gb_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "GbBankAccountService.CreateParams",
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Create a GB bank account.
        """
        return cast(
            GbBankAccount,
            await self._request_async(
                "post",
                "/v2/core/vault/gb_bank_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def initiate_confirmation_of_payee(
        self,
        id: str,
        params: "GbBankAccountService.InitiateConfirmationOfPayeeParams" = {},
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Initiate Confirmation of Payee (CoP) in order to verify that the owner of a UK bank account matches
        who you expect. This must be done on all UK bank accounts before sending domestic OutboundPayments. If
        the result is a partial match or a non match, explicit acknowledgement using AcknowledgeConfirmationOfPayee
        is required before sending funds.
        """
        return cast(
            GbBankAccount,
            self._request(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/initiate_confirmation_of_payee".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def initiate_confirmation_of_payee_async(
        self,
        id: str,
        params: "GbBankAccountService.InitiateConfirmationOfPayeeParams" = {},
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Initiate Confirmation of Payee (CoP) in order to verify that the owner of a UK bank account matches
        who you expect. This must be done on all UK bank accounts before sending domestic OutboundPayments. If
        the result is a partial match or a non match, explicit acknowledgement using AcknowledgeConfirmationOfPayee
        is required before sending funds.
        """
        return cast(
            GbBankAccount,
            await self._request_async(
                "post",
                "/v2/core/vault/gb_bank_accounts/{id}/initiate_confirmation_of_payee".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: "GbBankAccountService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Retrieve a GB bank account.
        """
        return cast(
            GbBankAccount,
            self._request(
                "get",
                "/v2/core/vault/gb_bank_accounts/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: "GbBankAccountService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> GbBankAccount:
        """
        Retrieve a GB bank account.
        """
        return cast(
            GbBankAccount,
            await self._request_async(
                "get",
                "/v2/core/vault/gb_bank_accounts/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2._amount import AmountParam
from stripe.v2._list_object import ListObject
from stripe.v2.money_management._outbound_transfer import OutboundTransfer
from typing import Dict, List, cast
from typing_extensions import Literal, NotRequired, TypedDict


class OutboundTransferService(StripeService):
    class CancelParams(TypedDict):
        pass

    _CreateParamsBase = TypedDict(
        "CreateParams",
        {"from": "OutboundTransferService.CreateParamsFrom"},
    )

    class CreateParams(_CreateParamsBase):
        amount: AmountParam
        """
        The "presentment amount" for the OutboundPayment.
        """
        delivery_options: NotRequired[
            "OutboundTransferService.CreateParamsDeliveryOptions"
        ]
        """
        Delivery options to be used to send the OutboundTransfer.
        """
        description: NotRequired[str]
        """
        An arbitrary string attached to the OutboundTransfer. Often useful for displaying to users.
        """
        metadata: NotRequired[Dict[str, str]]
        """
        Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
        """
        to: "OutboundTransferService.CreateParamsTo"
        """
        To which payout method to send the OutboundTransfer.
        """

    class CreateParamsDeliveryOptions(TypedDict):
        bank_account: NotRequired[Literal["automatic", "local", "wire"]]
        """
        Open Enum. Method for bank account.
        """

    class CreateParamsFrom(TypedDict):
        currency: str
        """
        Describes the FinancialAmount's currency drawn from.
        """
        financial_account: str
        """
        The FinancialAccount that funds were pulled from.
        """

    class CreateParamsTo(TypedDict):
        currency: NotRequired[str]
        """
        Describes the currency to send to the recipient.
        If included, this currency must match a currency supported by the destination.
        Can be omitted in the following cases:
        - destination only supports one currency
        - destination supports multiple currencies and one of the currencies matches the FA currency
        - destination supports multiple currencies and one of the currencies matches the presentment currency
        Note - when both FA currency and presentment currency are supported, we pick the FA currency to minimize FX.
        """
        payout_method: str
        """
        The payout method which the OutboundTransfer uses to send payout.
        """

    class ListParams(TypedDict):
        created: NotRequired[str]
        """
        Filter for objects created at the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        created_gt: NotRequired[str]
        """
        Filter for objects created after the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        created_gte: NotRequired[str]
        """
        Filter for objects created on or after the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        created_lt: NotRequired[str]
        """
        Filter for objects created before the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        created_lte: NotRequired[str]
        """
        Filter for objects created on or before the specified timestamp.
        Must be an RFC 3339 date & time value, for example: 2022-09-18T13:22:00Z.
        """
        limit: NotRequired[int]
        """
        The maximum number of results to return.
        """
        status: NotRequired[
            List[
                Literal[
                    "canceled", "failed", "posted", "processing", "returned"
                ]
            ]
        ]
        """
        Closed Enum. Only return OutboundTransfers with this status.
        """

    class RetrieveParams(TypedDict):
        pass

    def cancel(
        self,
        id: str,
        params: "OutboundTransferService.CancelParams" = {},
        options: RequestOptions = {},
    ) -> OutboundTransfer:
        """
        Cancels an OutboundTransfer. Only processing OutboundTransfers can be canceled.
        """
        return cast(
            OutboundTransfer,
            self._request(
                "post",
                "/v2/money_management/outbound_transfers/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        id: str,
        params: "OutboundTransferService.CancelParams" = {},
        options: RequestOptions = {},
    ) -> OutboundTransfer:
        """
        Cancels an OutboundTransfer. Only processing OutboundTransfers can be canceled.
        """
        return cast(
            OutboundTransfer,
            await self._request_async(
                "post",
                "/v2/money_management/outbound_transfers/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "OutboundTransferService.CreateParams",
        options: RequestOptions = {},
    ) -> OutboundTransfer:
        """
        Creates an OutboundTransfer.
        """
        return cast(
            OutboundTransfer,
            self._request(
                "post",
                "/v2/money_management/outbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "OutboundTransferService.CreateParams",
        options: RequestOptions = {},
    ) -> OutboundTransfer:
        """
        Creates an OutboundTransfer.
        """
        return cast(
            OutboundTransfer,
            await self._request_async(
                "post",
                "/v2/money_management/outbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        params: "OutboundTransferService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[OutboundTransfer]:
        """
        Returns a list of OutboundTransfers that match the provided filters.
        """
        return cast(
            ListObject[OutboundTransfer],
            self._request(
                "get",
                "/v2/money_management/outbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "OutboundTransferService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[OutboundTransfer]:
        """
        Returns a list of OutboundTransfers that match the provided filters.
        """
        return cast(
            ListObject[OutboundTransfer],
            await self._request_async(
                "get",
                "/v2/money_management/outbound_transfers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: "OutboundTransferService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> OutboundTransfer:
        """
        Retrieves the details of an existing OutboundTransfer by passing the unique OutboundTransfer ID from either the OutboundPayment create or list response.
        """
        return cast(
            OutboundTransfer,
            self._request(
                "get",
                "/v2/money_management/outbound_transfers/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: "OutboundTransferService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> OutboundTransfer:
        """
        Retrieves the details of an existing OutboundTransfer by passing the unique OutboundTransfer ID from either the OutboundPayment create or list response.
        """
        return cast(
            OutboundTransfer,
            await self._request_async(
                "get",
                "/v2/money_management/outbound_transfers/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2._amount import AmountParam
from stripe.v2._financial_address_credit_simulation import (
    FinancialAddressCreditSimulation,
)
from stripe.v2._financial_address_generated_microdeposits import (
    FinancialAddressGeneratedMicrodeposits,
)
from typing import cast
from typing_extensions import Literal, NotRequired, TypedDict


class FinancialAddressService(StripeService):
    class CreditParams(TypedDict):
        amount: AmountParam
        """
        Object containing the amount value and currency to credit.
        """
        network: Literal["ach", "fps", "rtp", "wire"]
        """
        Open Enum. The network to use in simulating the funds flow. This will be the reflected in the resulting ReceivedCredit.
        """
        statement_descriptor: NotRequired[str]
        """
        String explaining funds flow. Use this field to populate the statement descriptor of the ReceivedCredit created as an eventual result of this simulation.
        """

    class GenerateMicrodepositsParams(TypedDict):
        pass

    def credit(
        self,
        id: str,
        params: "FinancialAddressService.CreditParams",
        options: RequestOptions = {},
    ) -> FinancialAddressCreditSimulation:
        """
        Simulate crediting a FinancialAddress in a Sandbox environment. This can be used to add virtual funds and increase your balance for testing.
        """
        return cast(
            FinancialAddressCreditSimulation,
            self._request(
                "post",
                "/v2/test_helpers/financial_addresses/{id}/credit".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def credit_async(
        self,
        id: str,
        params: "FinancialAddressService.CreditParams",
        options: RequestOptions = {},
    ) -> FinancialAddressCreditSimulation:
        """
        Simulate crediting a FinancialAddress in a Sandbox environment. This can be used to add virtual funds and increase your balance for testing.
        """
        return cast(
            FinancialAddressCreditSimulation,
            await self._request_async(
                "post",
                "/v2/test_helpers/financial_addresses/{id}/credit".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def generate_microdeposits(
        self,
        id: str,
        params: "FinancialAddressService.GenerateMicrodepositsParams" = {},
        options: RequestOptions = {},
    ) -> FinancialAddressGeneratedMicrodeposits:
        """
        Generates microdeposits for a FinancialAddress in a Sandbox environment.
        """
        return cast(
            FinancialAddressGeneratedMicrodeposits,
            self._request(
                "post",
                "/v2/test_helpers/financial_addresses/{id}/generate_microdeposits".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def generate_microdeposits_async(
        self,
        id: str,
        params: "FinancialAddressService.GenerateMicrodepositsParams" = {},
        options: RequestOptions = {},
    ) -> FinancialAddressGeneratedMicrodeposits:
        """
        Generates microdeposits for a FinancialAddress in a Sandbox environment.
        """
        return cast(
            FinancialAddressGeneratedMicrodeposits,
            await self._request_async(
                "post",
                "/v2/test_helpers/financial_addresses/{id}/generate_microdeposits".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, Optional
from typing_extensions import Literal


class Transaction(StripeObject):
    """
    Use Transactions to view changes to your FinancialAccount balance over time. Every flow that moves money, such as OutboundPayments or ReceivedCredits, will have one or more Transactions that describes how the flow impacted your balance. Note that while the FinancialAccount balance will always be up to date, be aware that Transactions and TransactionEntries are created shortly after to reflect changes.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.transaction"]] = (
        "v2.money_management.transaction"
    )

    class BalanceImpact(StripeObject):
        available: Amount
        """
        Impact to the available balance.
        """
        inbound_pending: Amount
        """
        Impact to the inbound_pending balance.
        """
        outbound_pending: Amount
        """
        Impact to the outbound_pending balance.
        """

    class Flow(StripeObject):
        type: Literal[
            "adjustment",
            "currency_conversion",
            "fee_transaction",
            "inbound_transfer",
            "outbound_payment",
            "outbound_transfer",
            "received_credit",
            "received_debit",
        ]
        """
        Open Enum. Type of the flow that created the Transaction. The field matching this value will contain the ID of the flow.
        """
        adjustment: Optional[str]
        """
        If applicable, the ID of the Adjustment that created this Transaction.
        """
        fee_transaction: Optional[str]
        """
        If applicable, the ID of the FeeTransaction that created this Transaction.
        """
        inbound_transfer: Optional[str]
        """
        If applicable, the ID of the InboundTransfer that created this Transaction.
        """
        outbound_payment: Optional[str]
        """
        If applicable, the ID of the OutboundPayment that created this Transaction.
        """
        outbound_transfer: Optional[str]
        """
        If applicable, the ID of the OutboundTransfer that created this Transaction.
        """
        received_credit: Optional[str]
        """
        If applicable, the ID of the ReceivedCredit that created this Transaction.
        """
        received_debit: Optional[str]
        """
        If applicable, the ID of the ReceivedDebit that created this Transaction.
        """

    class StatusTransitions(StripeObject):
        posted_at: Optional[str]
        """
        The time at which the Transaction became posted. Only present if status == posted.
        """
        void_at: Optional[str]
        """
        The time at which the Transaction became void. Only present if status == void.
        """

    amount: Amount
    """
    The amount of the Transaction.
    """
    balance_impact: BalanceImpact
    """
    The delta to the FinancialAccount's balance. The balance_impact for the Transaction is equal to sum of its
    TransactionEntries that have `effective_at`s in the past.
    """
    category: Literal[
        "adjustment",
        "inbound_transfer",
        "outbound_payment",
        "outbound_transfer",
        "received_credit",
        "received_debit",
        "return",
        "stripe_fee",
    ]
    """
    Open Enum. A descriptive category used to classify the Transaction.
    """
    created: str
    """
    Time at which the object was created. Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    financial_account: str
    """
    Indicates the FinancialAccount affected by this Transaction.
    """
    flow: Flow
    """
    Details about the Flow object that created the Transaction.
    """
    id: str
    """
    Unique identifier for the object.
    """
    object: Literal["v2.money_management.transaction"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    status: Literal["pending", "posted", "void"]
    """
    Closed Enum. Current status of the Transaction.
    A Transaction is `pending` if either `balance_impact.inbound_pending` or `balance_impact.outbound_pending` is non-zero.
    A Transaction is `posted` if only `balance_impact.available` is non-zero.
    A Transaction is `void` if there is no balance impact.
    `posted` and `void` are terminal states, and no additional entries will be added to the Transaction.
    """
    status_transitions: StatusTransitions
    """
    Timestamps for when the Transaction transitioned to a particular status.
    """
    _inner_class_types = {
        "balance_impact": BalanceImpact,
        "flow": Flow,
        "status_transitions": StatusTransitions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class PayoutMethod(StripeObject):
    """
    Use the PayoutMethods API to list and interact with PayoutMethod objects.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.payout_method"]] = (
        "v2.money_management.payout_method"
    )

    class UsageStatus(StripeObject):
        payments: Literal["eligible", "invalid", "requires_action"]
        """
        Payments status.
        """
        transfers: Literal["eligible", "invalid", "requires_action"]
        """
        Transfers status.
        """

    class BankAccount(StripeObject):
        archived: bool
        """
        Whether this PayoutMethodBankAccount object was archived. PayoutMethodBankAccount objects can be archived through
        the /archive API, and they will not be automatically archived by Stripe. Archived PayoutMethodBankAccount objects
        cannot be used as payout methods and will not appear in the payout method list.
        """
        bank_name: str
        """
        The name of the bank this bank account is in. This field is populated automatically by Stripe.
        """
        country: str
        """
        The country code of the bank account.
        """
        enabled_delivery_options: List[str]
        """
        List of enabled flows for this bank account (wire or local).
        """
        last4: str
        """
        The last 4 digits of the account number.
        """
        routing_number: Optional[str]
        """
        The routing number of the bank account, if present.
        """
        supported_currencies: List[str]
        """
        The list of currencies supported by this bank account.
        """

    class Card(StripeObject):
        archived: bool
        """
        Whether the PayoutMethodCard object was archived. PayoutMethodCard objects can be archived through
        the /archive API, and they will not be automatically archived by Stripe. Archived PayoutMethodCard objects
        cannot be used as payout methods and will not appear in the payout method list.
        """
        exp_month: str
        """
        The month the card expires.
        """
        exp_year: str
        """
        The year the card expires.
        """
        last4: str
        """
        The last 4 digits of the card number.
        """

    available_payout_speeds: List[Literal["instant", "standard"]]
    """
    A set of available payout speeds for this payout method.
    """
    created: str
    """
    Created timestamp.
    """
    id: str
    """
    ID of the PayoutMethod object.
    """
    latest_outbound_setup_intent: Optional[str]
    """
    ID of the underlying active OutboundSetupIntent object, if any.
    """
    object: Literal["v2.money_management.payout_method"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    type: Literal["bank_account", "card"]
    """
    Closed Enum. The type of payout method.
    """
    usage_status: UsageStatus
    """
    Indicates whether the payout method has met the necessary requirements for outbound money movement.
    """
    bank_account: Optional[BankAccount]
    """
    The PayoutMethodBankAccount object details.
    """
    card: Optional[Card]
    """
    The PayoutMethodCard object details.
    """
    _inner_class_types = {
        "usage_status": UsageStatus,
        "bank_account": BankAccount,
        "card": Card,
    }
